<?php
/**
* Get the list of registered Post Types options.
*
* @since 3.18
* @since 4.0.7 Added the $require_editor parameter.
*
* @param boolean|callable $usort Comparision callback.
* @param boolean          $require_editor Optional. Whether to retrieve only post type that has editor support.
*
* @return array
*/
function df_get_registered_post_type_options( $usort = false, $require_editor = true ) {
   $require_editor_key = $require_editor ? '1' : '0';
   $key                = "df_get_registered_post_type_options:{$require_editor_key}";

   if ( ET_Core_Cache::has( $key ) ) {
       return ET_Core_Cache::get( $key );

   }

   $blocklist = et_builder_get_blocklisted_post_types();
   $allowlist = et_builder_get_third_party_post_types();

   // Extra and Library layouts shouldn't appear in Theme Options as configurable post types.
   /**
    * Get array of post types to prevent from appearing as options for builder usage.
    *
    * @since 4.0
    *
    * @param string[] $blocklist Post types to blocklist.
    */
   $blocklist      = array_merge(
       $blocklist,
       array(
           'et_pb_layout',
           'layout',
           'post',
           'attachment',
           'page'
       )
   );
   $blocklist      = apply_filters( 'et_builder_post_type_options_blocklist', $blocklist );
   $raw_post_types = get_post_types(
       array(
           'show_ui' => true,
       ),
       'objects'
   );
   $post_types     = array();

   foreach ( $raw_post_types as $post_type ) {
       $is_allowlisted  = in_array( $post_type->name, $allowlist, true );
       $is_blocklisted  = in_array( $post_type->name, $blocklist, true );
       $supports_editor = $require_editor ? post_type_supports( $post_type->name, 'editor' ) : true;
       $is_public       = et_builder_is_post_type_public( $post_type->name );

       if ( ! $is_allowlisted && ( $is_blocklisted || ! $supports_editor || ! $is_public ) ) {
           continue;
       }

       $post_types[] = $post_type;
   }

   if ( $usort && is_callable( $usort ) ) {
       usort( $post_types, $usort );
   }

   $post_type_options = array_combine(
       wp_list_pluck( $post_types, 'name' ),
       wp_list_pluck( $post_types, 'label' )
   );

   // did_action() actually checks if the action has started, not ended so we
   // need to check that we are not currently doing the action as well.
   if ( did_action( 'init' ) && ! doing_action( 'init' ) ) {
       // Only cache the value after init is done when we are sure all
       // plugins have registered their post types.
       ET_Core_Cache::add( $key, $post_type_options );
   }

   $post_type_options['select'] = 'Select Post Type';

   if(empty($post_type_options)) {
        $post_type_options = array(
            'not_found' => 'No Custom Post Type Found'
        );
   }

   return $post_type_options;
}


/**
 * CPT Image
 * 
 * @param Array $settings
 */
function df_cpt_image($settings = array(), $builder = false) {
    global $post;
    
    $module_class = isset($settings['module_vb_class']) ? $settings['module_vb_class'] : '';
    $size_array = array(
        'large' => array(1080, 675),
        'mid-hr' => array(350, 450),
        'mid' => array(400, 250),
        'mid-squ' => array(400, 400),
        'sm-squ' => array(300, 300),
        'default' => 'default'
    );

    $image_overlay = $settings['overlay'];
    $classes = '';
    $overlay = '';
    $overlay_icon = '';

    $post_format    = et_pb_post_format();
    $width          = (int) apply_filters( 'et_pb_blog_image_width', $size_array[$settings['image_size']][0] );
    $height         = (int) apply_filters( 'et_pb_blog_image_height', $size_array[$settings['image_size']][1] );

    $titletext      = get_the_title();
    $alttext        = get_post_meta( get_post_thumbnail_id(), '_wp_attachment_image_alt', true );
    $thumbnail      = get_thumbnail( $width, $height, 'df-cpt-image', $alttext, $titletext, false, 'Blogimage' );
    $thumb          = $thumbnail['thumb'];
    
    $post_thumbnail = $size_array[$settings['image_size']] === 'default' ? get_the_post_thumbnail() : 
        print_thumbnail( $thumb, $thumbnail['use_timthumb'], $titletext, $width, $height, '', false );

    // overlay
    $classes = sprintf('df-hover-effect %1$s',
        $settings['image_scale']
    );
    if($image_overlay === 'on') {
        if($settings['overlay_icon'] === 'on') {
            $overlay_icon = sprintf('<span class="df-icon-wrap">
                    <span class="df-icon-overlay %2$s">%1$s</span>
                </span>', 
                esc_attr(et_pb_process_font_icon($settings['overlay_font_icon'])),
                $settings['overlay_icon_reveal']
            );
        }
        $classes .= ' has_overlay';
        $overlay = '<span class="df-overlay"></span>';
    }

    
    et_divi_post_format_content();

    if ( 'video' === $post_format && false !== ( $first_video = et_get_first_video() ) ) :
        $video_overlay = has_post_thumbnail() ? sprintf(
            '<div class="et_pb_video_overlay" style="background-image: url(%1$s); background-size: cover;">
                <div class="et_pb_video_overlay_hover">
                    <a href="#" class="et_pb_video_play"></a>
                </div>
            </div>',
            $thumb
        ) : '';
        
        if(empty($first_video)) return;
        echo sprintf(
            '<div class="df-item-wrap df-cpt-image-wrap %3$s %4$s">
                <div class="et_main_video_container">
                    %1$s
                    %2$s
                </div>
            </div>',
            et_core_esc_previously( $video_overlay ),
            et_core_esc_previously( $first_video ),
            esc_attr($settings['class']),
            esc_attr($module_class)
        );
    else:
        if(!empty($thumb)) {
            echo sprintf('<div class="df-item-wrap df-cpt-image-wrap %3$s %7$s">
                    <a class="%4$s" href="%2$s">%1$s%5$s%6$s</a>
                </div>', 
                et_core_esc_previously($post_thumbnail), 
                esc_url(get_the_permalink()),
                esc_attr($settings['class']),
                esc_attr($classes),
                et_core_esc_previously($overlay),
                et_core_esc_previously($overlay_icon),
                esc_attr($module_class)
            );
        } elseif (empty($thumb) && $builder === true) {
            echo sprintf('<div class="df-item-wrap df-cpt-image-wrap df-empty-element %1$s %2$s"></div>', 
                esc_attr($settings['class']),
                esc_attr($module_class)
            );
        };
        
    endif;
}


/**
 * CPT Date
 * 
 * @param Array $settings
 */
function df_cpt_date($settings = array(), $builder = false) {
    global $post;


    $module_class = isset($settings['module_vb_class']) ? $settings['module_vb_class'] : '';

    echo sprintf('<span class="df-item-wrap df-cpt-date-wrap %2$s %4$s">%3$s %1$s</span>', 
        get_the_date($settings['date_format']),
        esc_attr($settings['class']),
        et_core_esc_previously(df_cpt_render_icon_image($settings)),
        esc_attr($module_class)
    );
}
/**
 * CPT Author
 * 
 * @param Array $settings
 */
function df_cpt_author($settings = array(), $builder = false) {
    global $post;

    $module_class = isset($settings['module_vb_class']) ? $settings['module_vb_class'] : '';

    $height = '';
    $min_height = '';

    if($settings['show_author_image'] === 'on') {
        $height = $settings['author_image_size'] . 'px';
        $min_height = ' style=height:'.$settings['author_image_size'].'px;';
    }

    $author_image = 'on' === $settings['show_author_image'] ? sprintf(
        '<a href="%2$s" class="author-image">%1$s</a> ',
        get_avatar( get_the_author_meta( 'ID' ), $settings['author_image_size'] ),
        get_author_posts_url( get_the_author_meta( 'ID' ) )
    ) : '';

    $author_link = 'on' === $settings['show_author_image'] && 'on' === $settings['hide_author_text'] ? 
        '' : et_pb_get_the_author_posts_link();

    echo sprintf('<span class="df-item-wrap df-cpt-author-wrap %2$s %6$s">%4$s %3$s %1$s</span>', 
        et_core_esc_previously($author_link),
        esc_attr($settings['class']),
        et_core_esc_previously($author_image),
        et_core_esc_previously(df_cpt_render_icon_image($settings)),
        et_core_esc_previously($min_height),
        esc_attr($module_class)
    );
}
/**
 * CPT Title
 * 
 * @param Array $settings
 */
function df_cpt_title($settings = array(), $builder = false) {    
    global $post;

    $module_class = isset($settings['module_vb_class']) ? $settings['module_vb_class'] : '';

    if(!empty(get_the_title())) {
        echo sprintf('<div class="df-item-wrap df-cpt-title-wrap %4$s %5$s">
                <%3$s class="df-cpt-title">
                    <a href="%2$s">
                        %1$s
                    </a>
                </%3$s>     
            </div>', 
            esc_html(get_the_title()), 
            esc_url(get_the_permalink()), 
            esc_attr($settings['title_tag']),
            esc_attr($settings['class']),
            esc_attr($module_class)
        );
    } elseif (empty(get_the_title()) && $builder === true) {
        echo sprintf('<div class="df-item-wrap df-cpt-title-wrap df-empty-element %1$s %2$s"></div>', 
            esc_attr($settings['class']),
            esc_attr($module_class)
        );
    }
    
}

/**
 * CPT Content
 * 
 * @param Array $settings
 * @param Boolean
 */
function df_cpt_content($settings = array(), $builder = false) {
    global $post;

    $post_content = et_strip_shortcodes( et_delete_post_first_video( get_the_content() ), true );

    $module_class = isset($settings['module_vb_class']) ? $settings['module_vb_class'] : '';

    echo '<div class="df-item-wrap df-cpt-content-wrap '.esc_attr($settings['class']) . ' ' . esc_attr($module_class) .'">';
        if('content' === $settings['post_content']) {
            global $more;

            // page builder doesn't support more tag, so display the_content() in case of post made with page builder
            if ( et_pb_is_pagebuilder_used( get_the_ID() ) ) {
                $more = 1; // phpcs:ignore WordPress.WP.GlobalVariablesOverride

                echo et_core_intentionally_unescaped( apply_filters( 'the_content', $post_content ), 'html' );

            } else {
                $more = null; // phpcs:ignore WordPress.WP.GlobalVariablesOverride
                echo et_core_intentionally_unescaped( apply_filters( 'the_content', et_delete_post_first_video( get_the_content( esc_html__( 'read more...', 'et_builder' ) ) ) ), 'html' );
            }
        } elseif('excerpt' === $settings['post_content']) {
            if ( has_excerpt() && 'on' === $settings['use_post_excrpt']) {
                the_excerpt();
            } else {
                echo et_core_intentionally_unescaped( wpautop( et_delete_post_first_video( strip_shortcodes( truncate_post( intval($settings['excerpt_length']), false, '', true ) ) ) ), 'html' );
            }
        }
    echo '</div>';
}

/**
 * CPT Read More Button
 * 
 * @param Array $settings
 * @param Boolean
 * @return void
 */
function df_cpt_button($settings = array(), $builder = false) {
    global $post;

    $module_class = isset($settings['module_vb_class']) ? $settings['module_vb_class'] : '';

    $output = sprintf('<div class="df-item-wrap df-cpt-button-wrap %2$s %5$s">
            <a class="df-cpt-read-more" href="%1$s"><span>%3$s</span> %4$s</a>
        </div>', 
        get_the_permalink(),
        esc_attr($settings['class']),
        esc_html($settings['read_more_text']),
        df_cpt_render_icon_image($settings),
        esc_attr($module_class)
    );
    
    echo et_core_esc_previously($output);
}

/**
 * CPT element divider
 * 
 * @param Array $settings
 * @param Boolean
 * @return void
 */
function df_cpt_divider($settings = array(), $builder = false) {
    global $post;

    $module_class = isset($settings['module_vb_class']) ? $settings['module_vb_class'] : '';

    echo sprintf(
        '<div class="df-item-wrap %1$s %2$s">
            <span class="df-cpt-ele-divider"></span>
        </div>',
        esc_attr($settings['class']),
        esc_attr($module_class)
    );
}

/**
 * CPT Custom Text
 * 
 * @param Array $settings
 * @param Boolean
 */
function df_cpt_custom_text($settings = array(), $builder = false) {
    global $post;

    if( $settings['custom_text'] === '' ) return; 
    $module_class = isset($settings['module_vb_class']) ? $settings['module_vb_class'] : '';

    echo sprintf('<span class="df-item-wrap df-cpt-custom-text %2$s %3$s">%1$s</span>', 
        esc_html__($settings['custom_text'], 'divi_flash'),
        esc_attr($settings['class']),
        esc_attr($module_class)
    );
}

/**
 * CPT taxonomies
 * 
 * @param Array $settings
 * @param Boolean 
 * @return String
 */
function df_cpt_taxonomy($settings = array(), $builder = false) {
    global $post;

    $module_class = isset($settings['module_vb_class']) ? $settings['module_vb_class'] : '';

    if ($settings['taxonomy'] === 'select_tax') return;
    
    $taxonomies = get_the_term_list($post->ID, $settings['taxonomy'], '', ', ');

    if(!empty($taxonomies)) {
        echo sprintf('<span class="df-item-wrap df-cpt-taxonomies %1$s %2$s">
                %4$s %3$s
            </span>',
            esc_attr($settings['class']),
            esc_attr($module_class),
            wp_kses_post($taxonomies),
            et_core_esc_previously(df_cpt_render_icon_image($settings))
        );
    } elseif(empty($taxonomies) && $builder === true) {
        echo sprintf('<span class="df-item-wrap df-cpt-taxonomies df-empty-element %1$s %2$s"></span>',
            esc_attr($settings['class']),
            esc_attr($module_class)
        );
    }
    

}

/**
 * CPT image as background
 * 
 * @param String $use_image_as_background
 * @return String
 */
function df_cpt_image_as_background($use_image_as_background) {
    global $post;
    // use_image_as_background
    if($use_image_as_background === 'on' && !empty(get_the_post_thumbnail_url($post->ID))) {
        return sprintf('style="background-image:url(%1$s); 
            background-size: cover; 
            background-repeat: no-repeat;
            background-position: center;
            background-origin: border-box;"', 
            get_the_post_thumbnail_url($post->ID)
        );
    }
}

/**
 * Render markup for acf fields
 * 
 * Supported fields: 'text', 'number', 'textarea', 'range', 'email', 'url', 'image', 'select', 'date_picker', 'wysiwyg'
 * 
 * @param Array $settings
 * @param Boolean
 * @return String
 */
function df_cpt_acf_fields($settings = array(), $builder = false) {
    global $post;
    // get acf data stored in this instance
    // If no instance found then create instance and store all fields data
    $fields_storage = Df_Acf_Fields::getInstance();
    if(!isset($settings['acf_field'])) return;
    $field_type =  !empty($fields_storage->acf_fields_type) && isset($fields_storage->acf_fields_type[$settings['acf_field']]) ? 
        $fields_storage->acf_fields_type[$settings['acf_field']] : array();
    $module_class = isset($settings['module_vb_class']) ? $settings['module_vb_class'] : '';
    $default_value = '';

    if( class_exists('ACF') ) {
        $default_value = "No ACF field selected";
    }
    
    echo sprintf('<div class="df-item-wrap %1$s %2$s">', esc_attr($settings['class']), esc_attr($module_class));
        switch($field_type) {
            case 'text':
                echo et_core_esc_previously(df_acf_render_text_type($settings));
                break;
            case 'number':
                echo et_core_esc_previously(df_acf_render_number_type($settings));
                break;
            case 'textarea':
                echo et_core_esc_previously(df_acf_render_textarea_type($settings));
                break;
            case 'range':
                echo et_core_esc_previously(df_acf_render_range_type($settings));
                break;
            case 'email':
                echo et_core_esc_previously(df_acf_render_email_type($settings));
                break;
            case 'url':
                echo et_core_esc_previously(df_acf_render_url_type($settings));
                break;
            case 'image':
                echo et_core_esc_previously(df_acf_render_image_type($settings));
                break;
            case 'select':
                echo et_core_esc_previously(df_acf_render_select_type($settings));
                break;
            case 'date_picker':
                echo et_core_esc_previously(df_acf_render_date_type($settings));
                break;
            case 'wysiwyg':
                echo et_core_esc_previously(df_acf_render_wysiwyg_type($settings));
                break;
            default:
                echo esc_html($default_value);
        }
    echo '</div>';
}

/**
 * Render ACF text field
 * 
 * Supported tag - br, em, strong, b, p, ul, ol, li
 * 
 * @param Array $settings
 * @return String
 */
function df_acf_render_text_type($settings) {
    global $post;
    $acf_field_data = wp_kses(get_field( $settings['acf_field'], $post->ID ), df_allowed_html_for_text_input());

    return $acf_field_data !== '' ?
        sprintf('%4$s%1$s%2$s%3$s',
            df_acf_before_after($settings)['before'], 
            $acf_field_data, 
            df_acf_before_after($settings)['after'],
            df_cpt_render_icon_image($settings)
        ): '';
}
/**
 * Render ACF Number field
 * 
 * @param Array $settings
 * @return String
 */
function df_acf_render_number_type($settings) {
    global $post;
    $acf_field_data = wp_kses(get_field( $settings['acf_field'], $post->ID ), df_allowed_html_for_text_input());

    return $acf_field_data !== '' ? 
        sprintf('%4$s%1$s%2$s%3$s',
            df_acf_before_after($settings)['before'], 
            $acf_field_data, 
            df_acf_before_after($settings)['after'],
            df_cpt_render_icon_image($settings)
        ) : '';
}
/**
 * Render ACF textarea field
 * 
 * @param Array
 * @return String
 */
function df_acf_render_textarea_type($settings) {
    global $post;
    $acf_field_data = wp_kses(get_field( $settings['acf_field'], $post->ID ), df_allowed_html_for_text_input());

    return $acf_field_data !== '' ?
            sprintf('%4$s%1$s%2$s%3$s',
            df_acf_before_after($settings)['before'], 
            $acf_field_data, 
            df_acf_before_after($settings)['after'],
            df_cpt_render_icon_image($settings)
        ) : '';
}
/**
 * Render ACF range field
 * 
 * @param Array
 * @return String
 */
function df_acf_render_range_type($settings) {
    global $post;
    $acf_field_data = wp_kses(get_field( $settings['acf_field'], $post->ID ), df_allowed_html_for_text_input());

    return $acf_field_data !== '' ?
            sprintf('%4$s%1$s%2$s%3$s',
            df_acf_before_after($settings)['before'], 
            $acf_field_data, 
            df_acf_before_after($settings)['after'],
            df_cpt_render_icon_image($settings)
        ) : '';
}
/**
 * Render ACF email field value
 * 
 * @param Array
 * @return String
 */
function df_acf_render_email_type($settings) {
    global $post;
    $acf_field_data = esc_attr(get_field( $settings['acf_field'], $post->ID ));
    $email_text = $settings['acf_email_text'] !== '' ?
        esc_html($settings['acf_email_text']) : $acf_field_data;

    if($acf_field_data !== '') {
        $acf_field_data = sprintf('%5$s%1$s<a href="mailto:%2$s">%4$s</a>%3$s', 
            df_acf_before_after($settings)['before'],
            $acf_field_data,
            df_acf_before_after($settings)['after'],
            $email_text,
            df_cpt_render_icon_image($settings)
        );
    }

    return $acf_field_data;
}
/**
 * Render ACF URL field type
 * 
 * @param Array
 * @return String
 */
function df_acf_render_url_type($settings) {
    global $post;
    $acf_field_data = esc_attr(get_field($settings['acf_field'], $post->ID));
    $url_text = $settings['acf_url_text'] !== '' ?
        esc_html($settings['acf_url_text']) : $acf_field_data;
    $url_target = $settings['acf_url_new_window'] === 'on' ?
        'target="_blank"' : '';

    if($acf_field_data !== '') {
        $acf_field_data = sprintf('%6$s%1$s<a href="%2$s" %5$s>%4$s</a>%3$s',
            df_acf_before_after($settings)['before'],
            $acf_field_data,
            df_acf_before_after($settings)['after'],
            $url_text,
            $url_target,
            df_cpt_render_icon_image($settings)
        );
    }

    return $acf_field_data;
}
/**
 * Render ACF Image field type
 * 
 * @param Array
 * @return String
 */
function df_acf_render_image_type($settings) {
    global $post;
    $acf_image_url = get_field($settings['acf_field'], $post->ID);

    if($acf_image_url !== '') {
        return sprintf('%4$s%1$s<img class="df-acf-image" src="%2$s" />%3$s', 
            df_acf_before_after($settings)['before'],
            esc_attr($acf_image_url['url']),
            df_acf_before_after($settings)['after'],
            df_cpt_render_icon_image($settings)
        );
    }
}
/**
 * Render ACF Select Field
 * 
 * @param Array
 * @return String
 */
function df_acf_render_select_type($settings) {
    global $post;

    $acf_field_data = wp_kses(get_field( $settings['acf_field'], $post->ID ), df_allowed_html_for_text_input());

    return $acf_field_data !== '' ?
            sprintf('%4$s%1$s%2$s%3$s',
            df_acf_before_after($settings)['before'], 
            $acf_field_data, 
            df_acf_before_after($settings)['after'],
            df_cpt_render_icon_image($settings)
        ) : '';
}
/**
 * Render ACF Date Picker Field
 * 
 * @param Array
 * @return String
 */
function df_acf_render_date_type($settings) {
    global $post;

    $acf_field_data = wp_kses(get_field( $settings['acf_field'], $post->ID ), df_allowed_html_for_text_input());

    return $acf_field_data !== '' ?
            sprintf('%4$s%1$s%2$s%3$s',
            df_acf_before_after($settings)['before'], 
            $acf_field_data, 
            df_acf_before_after($settings)['after'],
            df_cpt_render_icon_image($settings)
        ) : '';
}
/**
 * Render ACF wysiwyg Field
 * 
 * @param Array
 * @return String
 */
function df_acf_render_wysiwyg_type($settings) {
    global $post;

    $acf_field_data = wp_kses_post(get_field( $settings['acf_field'], $post->ID));

    return $acf_field_data !== '' ?
            sprintf('%4$s%1$s%2$s%3$s',
            df_acf_before_after($settings)['before'], 
            $acf_field_data, 
            df_acf_before_after($settings)['after'],
            df_cpt_render_icon_image($settings)
        ) : '';
}

/**
 * Render ACF Fields before and after text
 * 
 * @param Array $settings
 * @return Array
 */
function df_acf_before_after($settings){
    $before_label = wp_kses($settings['acf_before_label'], df_allowed_html_for_text_input());
    $after_label = wp_kses($settings['acf_after_label'], df_allowed_html_for_text_input());

    return array(
        'before' => $before_label !== '' ? sprintf('<span class="before-text">%1$s</span>', $before_label) : '',
        'after' => $after_label !== '' ? sprintf('<span class="after-text">%1$s</span>', $after_label) : ''
    );
}
/**
 * Render Posts for BlogGrid Module on VB
 * 
 */
add_action('wp_ajax_df_cpt_grid', 'df_cpt_grid');
function df_cpt_grid() {
    global $paged, $post, $wp_query, $wp_filter, $__et_blog_module_paged;

    // create the display gallery code
    $data = json_decode(file_get_contents('php://input'), true);
    if (! wp_verify_nonce( $data['et_admin_load_nonce'], 'et_admin_load_nonce' )) {
        wp_die();
    }

    $cpt_items = isset($data['cptItems']) ? $data['cptItems'] : array();
    
    $cpt_item_inner = $cpt_items['inner'];
    $cpt_item_outer = $cpt_items['outer'];

    $post_type = isset($data['post_type']) ? $data['post_type'] : ' project';
    $posts_number = isset($data['posts_number']) ? intval($data['posts_number']) : 10;
    $post_display = isset($data['post_display']) ? $data['post_display'] : 'recent';
    $orderby = isset($data['orderby']) ? $data['orderby'] : '1';
    $selected_taxonomy = isset($data['selected_taxonomy']) ? $data['selected_taxonomy'] : '';
    $selected_terms = isset($data['selected_terms']) ? $data['selected_terms'] : '';
    $offset_number = isset($data['offset_number']) ? $data['offset_number']: '0';
    $layout = isset($data['layout']) ? $data['layout'] : 'grid';
    $use_image_as_background = isset($data['use_image_as_background']) ? $data['use_image_as_background'] : 'off';
    $use_background_scale = isset($data['use_background_scale']) ? $data['use_background_scale'] : 'off';
    $equal_height = isset($data['equal_height']) ? $data['equal_height'] : 'off';
    $show_pagination = isset($data['show_pagination']) ? $data['show_pagination'] : 'off';
    $use_number_pagination = isset($data['use_number_pagination']) ? $data['use_number_pagination'] : 'off';
    $older_text = isset($data['older_text']) ? $data['older_text'] : 'Older Entries';
    $newer_text = isset($data['newer_text']) ? $data['newer_text'] : 'Next Entries';
    $use_current_loop = $data['use_current_loop'];
    $post_type_arch = $data['post_type_arch'];

    $post_type = $use_current_loop === 'on' ? $post_type_arch : $post_type;

    $query_args = array(
        'posts_per_page' => $posts_number,
        'post_status'    => array( 'publish' ),
        'post_type'      => $post_type,
    );

    // by taxonomy
    if( 'by_tax' === $post_display && '' !== $selected_terms ) {
        $query_args['tax_query'] = array( //phpcs:ignore WordPress.DB.SlowDBQuery
            'relation' => 'AND',
            array(
                'taxonomy'  => $selected_taxonomy,
                'field'     => 'term_id',
                'terms'     => explode(',', $selected_terms)
            )
        );
    }
    // orderby
    if ( 'recent' == $post_display) {
        if ( '3' === $orderby ) {
            $query_args['orderby'] = 'rand';
        } else if('2' === $orderby) {
            $query_args['orderby'] = 'date';
            $query_args['order'] = 'ASC';
        } else {
            $query_args['orderby'] = 'date';
            $query_args['order'] = 'DESC';
        }
    }

    $df_pg_paged = is_front_page() ? get_query_var( 'page' ) : get_query_var( 'paged' );
    if ( is_front_page() ) {
        $paged = $df_pg_paged; //phpcs:ignore WordPress.WP.GlobalVariablesOverride
    }
    if ( $__et_blog_module_paged > 1 ) {
        $df_pg_paged            = $__et_blog_module_paged;
        $paged                  = $__et_blog_module_paged; //phpcs:ignore WordPress.WP.GlobalVariablesOverride
        $query_args['paged']    = $__et_blog_module_paged;
    }
    if ( ! is_search() ) {
        $paged = $df_pg_paged; //phpcs:ignore WordPress.WP.GlobalVariablesOverride
        $query_args['paged'] = $df_pg_paged;
    }
    $query_args['paged'] = $df_pg_paged;

    if ( '' !== $offset_number && ! empty( $offset_number ) ) {
        /**
         * Offset + pagination don't play well. Manual offset calculation required
         *
         * @see: https://codex.wordpress.org/Making_Custom_Queries_using_Offset_and_Pagination
         */
        if ( $paged > 1 ) {
            $query_args['offset'] = ( ( $df_pg_paged - 1 ) * intval( $posts_number ) ) + intval( $offset_number );
        } else {
            $query_args['offset'] = intval( $offset_number );
        }
    }
    
    ob_start();
    query_posts( $query_args ); // phpcs:ignore WordPress.WP.DiscouragedFunctions

    echo '<div class="df-cpts-wrap layout-'.esc_attr($layout).'">';
    if ( have_posts() ) {
        while ( have_posts() ) {
            the_post();

            $equal_height_class = $equal_height === 'on' ? ' df-equal-height' : '';
            ?>
            <article id="post-<?php the_ID(); ?>" <?php post_class( "df-cpt-item v2{$equal_height_class}" ) ?>>
                <div class="df-cpt-outer-wrap df-hover-trigger" <?php echo $use_background_scale !== 'on' ? et_core_esc_previously(df_cpt_image_as_background($use_image_as_background)): '';?>>
                    <?php 
                        // render markup to achive the scale effect.
                        if($use_image_as_background === 'on' && $use_background_scale === 'on') {
                            echo '<div class="df-cpt-bg-on-hover"><div ' . et_core_esc_previously(df_cpt_image_as_background($use_image_as_background)) .'></div></div>';
                        }
                        if(!empty($cpt_item_outer)) {
                            foreach( $cpt_item_outer as $cpt_item ) {

                                if( !isset($cpt_item['type'])) {
                                    continue;
                                }

                                $callback = 'df_cpt_' . $cpt_item['type'];

                                call_user_func($callback, $cpt_item, true);

                            } // end of foreach
                        }
                    ?>
                    <div class="df-cpt-inner-wrap">
                        <?php 
                            foreach( $cpt_item_inner as $cpt_item ) {

                                if( !isset($cpt_item['type'])) {
                                    continue;
                                }

                                $callback = 'df_cpt_' . $cpt_item['type'];

                                call_user_func($callback, $cpt_item, true);

                            } // end of foreach
                        ?>
                    </div>
                </div>
            </article>
            <?php
        } // endwhile
    }
    echo '</div>';

    // ajax navigation
    if ( 'on' === $show_pagination ) {
        if ( function_exists( 'wp_pagenavi' ) ) {
            wp_pagenavi();
        } else {
            add_filter( 'get_pagenum_link', array( 'DIFL_CptGrid', 'filter_pagination_url' ) );
            if ($use_number_pagination !== 'on') {
                cpt_render_pagination($older_text, $newer_text, true);
            } else {
                cpt_render_number_pagination($older_text, $newer_text, true);
            }
            remove_filter( 'get_pagenum_link', array( 'DIFL_CptGrid', 'filter_pagination_url' ) );
        }
    }

    wp_reset_query(); // phpcs:ignore WordPress.WP.DiscouragedFunctions
    $posts = ob_get_contents();
    ob_end_clean();

    if(empty($cpt_items)) {
        $posts = '<h2>Please create post element.</h3>';
    }

    wp_send_json_success($posts);
}

function df_display_cpt_grid_load_actions( $actions ) {
	$actions[] = 'df_cpt_grid';

	return $actions;
}
add_filter( 'et_builder_load_actions', 'df_display_cpt_grid_load_actions' );


/**
 * Render simple pagination for blog
 * 
 * @param String older entries text
 * 
 * @param String next entries text
 */
function cpt_render_pagination( $older, $newer, $builder = false ) {
    add_filter('next_posts_link_attributes', 'df_cpt_next_link_attributes');
    add_filter('previous_posts_link_attributes', 'df_cpt_prev_link_attributes');

    echo '<div class="df-pagination pagination clearfix">';
        if($builder === true) {
            echo '<a class="older page-numbers" href="#">'.esc_html__("{$older}",'divi_flash').'</a>';
            echo '<a class="newer page-numbers" href="#">'.esc_html__("{$newer}",'divi_flash').'</a>';
        } else {
            next_posts_link(esc_html__("{$older}",'divi_flash'));
            previous_posts_link(esc_html__("{$newer}", 'divi_flash'));
        }
    echo '</div>';

    remove_filter('next_posts_link_attributes', 'df_cpt_next_link_attributes');
    remove_filter('previous_posts_link_attributes', 'df_cpt_prev_link_attributes');
}
/**
 * Render numbered pagination
 * 
 * @param String older entries text
 * 
 * @param String next entries text
 */
function cpt_render_number_pagination( $older, $newer, $builder = false ) {
    global $wp_query;
    $big = 9999999; // need an unlikely integer
    echo '<div class="df-pagination pagination clearfix">';
        if($builder === true) {
            echo '<a class="prev page-numbers" href="#">'.esc_html__("{$older}",'divi_flash').'</a>';
        }
        echo paginate_links( array(
            'base' => str_replace( $big, '%#%', esc_url( get_pagenum_link( $big ) ) ),
            'format' => '?paged=%#%',
            'current' => max( 1, get_query_var('paged') ),
            'total' => $wp_query->max_num_pages,
            'prev_text' => esc_html__("{$older}",'divi_flash'),
            'next_text' => esc_html__("{$newer}", 'divi_flash')
        ));
    echo '</div>';
}
/**
 * add class attributes to
 * previous and next posts links
 * 
 */
function df_cpt_next_link_attributes() {
    return 'class="older page-numbers"';
}
function df_cpt_prev_link_attributes() {
    return 'class="newer page-numbers"';
}

/**
 * Render icon/image for times
 * 
 * @param Array $settings
 * @return String html
 */
function df_cpt_render_icon_image($settings) {
    if(('on' != $settings['use_icon']) && !isset($settings['image_icon'])) {
        return;
    }
    $image_icon = $settings['image_icon'] !== '' ? sprintf('<img class="df-icon-image" src="%1$s" />', esc_attr($settings['image_icon'])) : '';
    return $settings['use_icon'] === 'on' ? 
        sprintf('<span class="et-pb-icon">%1$s</span>', $settings['font_icon']) : $image_icon;
}

/**
 * Filter the main query paged arg to avoid pagination clashes with the Post modules pagination.
 *
 * @param WP_Query $query Query object.
 *
 * @return void
 */
function df_filter_main_query_paged_for_cpt_module( $query ) {
	global $__et_blog_module_paged;

	// phpcs:ignore WordPress.Security.NonceVerification -- This function does not change any state, and is therefore not susceptible to CSRF.
	if ( isset( $_GET['df_cpt'] ) && $query->is_main_query() ) {
		$__et_blog_module_paged = $query->get( 'paged' );
		$query->set( 'paged', 0 );
	}
}
add_filter( 'pre_get_posts', 'df_filter_main_query_paged_for_cpt_module' );
